@@ -93,6 +93,17 @@ func newNamespace(name string) *corev1.Namespace {
 	}
 }
 
+func newNode(name string) *corev1.Node {
+	return &corev1.Node{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: name,
+			Labels: map[string]string{
+				"node-role.kubernetes.io/master": "",
+			},
+		},
+	}
+}
+
 func newTestController(klusterlet *opratorapiv1.Klusterlet, appliedManifestWorks []runtime.Object, objects ...runtime.Object) *testController {
 	fakeKubeClient := fakekube.NewSimpleClientset(objects...)
 	fakeAPIExtensionClient := fakeapiextensions.NewSimpleClientset()
@@ -162,12 +173,12 @@ func ensureRegistrationDeployment(t *testing.T, deployment *appsv1.Deployment, s
 		t.Errorf("Expect 1 containers in deployment spec, actual %d", len(deployment.Spec.Template.Spec.Containers))
 	}
 	args := deployment.Spec.Template.Spec.Containers[0].Args
-	if serverURL == "" && len(args) != 4 {
-		t.Errorf("Expect 4 args in container spec, actual %d", len(args))
-	}
-	if serverURL != "" && len(deployment.Spec.Template.Spec.Containers[0].Args) != 5 {
+	if serverURL == "" && len(args) != 5 {
 		t.Errorf("Expect 5 args in container spec, actual %d", len(args))
 	}
+	if serverURL != "" && len(deployment.Spec.Template.Spec.Containers[0].Args) != 6 {
+		t.Errorf("Expect 6 args in container spec, actual %d", len(args))
+	}
 	clusterNameArg := ""
 	serverURLArg := ""
 	for _, arg := range args {
@@ -340,8 +351,8 @@ func TestSyncDelete(t *testing.T) {
 		}
 	}
 
-	if len(deleteCRDActions) != 1 {
-		t.Errorf("Expected 1 delete actions, but got %d", len(deleteCRDActions))
+	if len(deleteCRDActions) != 2 {
+		t.Errorf("Expected 2 delete actions, but got %d", len(deleteCRDActions))
 	}
 
 	updateWorkActions := []clienttesting.UpdateActionImpl{}
